@ECHO OFF
REM Licensed Materials - Property of IBM
REM
REM (C) Copyright IBM Corp. 2011, 2014 All Rights Reserved
REM
REM US Government Users Restricted Rights - Use, duplicate or
REM disclosure restricted by GSA ADP Schedule Contract with
REM IBM Corp.

REM Usage: This script is used to migrate the Storage Control DB2
REM  database.  It must be run using db2cmd instead of cmd.exe because
REM  db2cmd.exe initializes the DB2 environment for commands.
REM 
REM The file uses one parameter, a filename that is read to 
REM  set the following parameters (specified as SET PARAMETER=VALUE in the file
REM  we read in):
REM  
REM MIGRATION_OPERATION=backup
REM   MIGRATION_OPERATION can be either backup or restore
REM DB_NAME=TPCDB
REM   The name of the database we will backup or restore
REM BACKUP_DIR=c:\somelocation\backupdir
REM   The file system location where we will write the backup file to backup or 
REM    read the backup file to restore
REM MIGRATE_INFO_FILE=c:/test_backup/restore.info
REM   The filename we tell DB2 to write backup information to during the backup.
REM     We use forward slashes for passing to DB2.  If we need to pass it to a Windows
REM     command we need to convert them to backslashes using something similar 
REM     to "SET MIGRATION_INFO_FILE_WIN=%MIGRATION_INFO_FILE:/=\%"
REM DB2_BIN_PATH="C:\Program Files\IBM\SQLLIB"
REM   The path to the DB2 install to use for calling the db2 command. 
REM DB2_AUTH_FILE=c:\somelocation\backupdir\dbauth.info
REM   The path to a file that specifies the authentication information
REM    for DB2.  We split this from the file with the other parameters so we can
REM    restrict access to it since the authentication information is in clear text.
REM TIMESTAMP=
REM   The DB2 commands for restoring will use the most recent file by default.
REM   Part of the filename is a timestamp.  If we want to specify a specific backup
REM   to restore, we specify its timestamp.

REM DB2_AUTH_FILE has the following parameters:
REM DB_USER=dbUser
REM   The DB2 user to use for SQL commands.
REM DB_PASSWORD=dbPassword
REM   The DB2 password to use for SQL commands.

SETLOCAL ENABLEEXTENSIONS 

SET RETCODE=0
SET MIGRATION_PARAM_FILE="%1"
SET BACKUP_DIR=c:\temp\sc_backup

REM The parameter file must end in .bat or .cmd to avoid an error stating the
REM  file is not recognized as an internal or external command
CALL "%MIGRATION_PARAM_FILE%"

REM Read authentication information from a restricted file.  
REM   DB2_AUTH_FILE should be set in MIGRATION_PARAM_FILE
REM CALL "%DB2_AUTH_FILE%"

REM We're setting DB2 to store our SQL command results in a file
REM  and need to make sure the exists to prevent a DB2 error.
REM 2014: set DB2OPTIONS to be null as the file name path can't have space, have to add the -z param into db2 command
SET DB2OPTIONS=

REM Since we run this using db2cmd.exe on Windows, we need to start
REM  our SQL command sequences with DB2.
SET DB2_CMD=DB2 -z "%MIGRATE_INFO_FILE%"

SET migrationReturnCode=0

SET DIRHOME=

call :initializeJava

SET LOG=%DIRHOME%\log\SCMigDBSupport.log

GOTO Main


:CleanUpFiles
    REM For reach run, we're resetting the log file to an empty 
    REM  file because DB2 appends to the file.  
    TYPE NUL > "%MIGRATE_INFO_FILE%"
    GOTO %Return%

:PrepareForMigration
    echo "%DB2_CMD% connect TO %DB_NAME%"
    %DB2_CMD% CONNECT TO %DB_NAME% USER %DB_USER% USING %DB_PASSWORD% >> %LOG%
    IF ERRORLEVEL 1 ( 
       echo "Error while connecting to the database." >> %LOG% 2>&1
       EXIT /B 1 
    )
 
    echo "Quiesce database ..." >> %LOG%
    %DB2_CMD% QUIESCE DATABASE IMMEDIATE FORCE CONNECTIONS >> %LOG%
    IF ERRORLEVEL 1 ( 
       echo "Error while quiesceing the database." >> %LOG% 2>&1
       EXIT /B 1 
    )

    %DB2_CMD% CONNECT RESET >> %LOG%
    IF ERRORLEVEL 1 ( 
       echo "Error while resetting the connection to the database." >> %LOG% 2>&1
       EXIT /B 1 
    )


    GOTO %Return%

:BackupDatabase
	echo "Database backup directory: %BACKUP_DIR%"  >> %LOG%

	echo "Getting tablespace ids..." >> %LOG%
	%DB2_CMD% "connect to TPCDB" >> %LOG%
        IF ERRORLEVEL 1 ( 
           echo "Error while connecting to the database." >> %LOG% 2>&1
           EXIT /B 1 
        )

	%DB2_CMD% "select tbspace, tbspaceid from syscat.tablespaces" 2>&1 > "%BACKUP_DIR%"\Tablespaces.txt
        IF ERRORLEVEL 1 ( 
           echo "Error while getting tablespace ids from the database." >> %LOG% 2>&1
           EXIT /B 1 
        )

    if NOT EXIST "%BACKUP_DIR%" (
       mkdir "%BACKUP_DIR%"
    )
	echo "Backing up database %DB_NAME% ..." >> %LOG%
        %DB2_CMD% BACKUP DATABASE %DB_NAME% USER %DB_USER% USING %DB_PASSWORD% TO \"%BACKUP_DIR%\" WITH 2 BUFFERS BUFFER 1024 PARALLELISM 1 COMPRESS WITHOUT PROMPTING >> %LOG%
	SET migrationReturnCode=%ERRORLEVEL%
	echo "Database backup finished with exit code %migrationReturnCode%, check %MIGRATE_INFO_FILE% for the details" >> %LOG%
    GOTO %Return%
    
:RestoreDatabase
	echo "Database backup directory: %BACKUP_DIR%" >> %LOG%
        echo "Getting Tablespace ids..." >> %LOG%
    
	SET TPCTBSPKEYSID=-1
	SET TPCTBSPPMID=-1
	SET TPCTBSPTEMPID=-1
	SET TPCTBSPDATAID=-1
	SET TPCTBSPUSRTMPID=-1


	for /f "usebackq tokens=1,2 delims= " %%a in (`type "%BACKUP_DIR%"\Tablespaces.txt`) do (
		if "%%a"=="TPCTBSPKEYS" ( set TPCTBSPKEYSID=%%b)
	)

	for /f "usebackq tokens=1,2 delims= " %%a in (`type "%BACKUP_DIR%"\Tablespaces.txt`) do (
		if "%%a"=="TPCTBSPPM" ( set TPCTBSPPMID=%%b)
	)

	for /f "usebackq tokens=1,2 delims= " %%a in (`type "%BACKUP_DIR%"\Tablespaces.txt`) do (
		if "%%a"=="TPCTBSPTEMP" ( set TPCTBSPTEMPID=%%b)
	)

	for /f "usebackq tokens=1,2 delims= " %%a in (`type "%BACKUP_DIR%"\Tablespaces.txt`) do (
		if "%%a"=="TPCTBSPDATA" ( set TPCTBSPDATAID=%%b)
	)

	for /f "usebackq tokens=1,2 delims= " %%a in (`type "%BACKUP_DIR%"\Tablespaces.txt`) do (
		if "%%a"=="TPCTBSPUSRTMP" ( set TPCTBSPUSRTMPID=%%b)
	)


	echo TPCTBSPKEYSID=%TPCTBSPKEYSID% >> %LOG%
	echo TPCTBSPPMID=%TPCTBSPPMID% >> %LOG%
	echo TPCTBSPTEMPID=%TPCTBSPTEMPID% >> %LOG%
	echo TPCTBSPDATAID=%TPCTBSPDATAID% >> %LOG%
	echo TPCTBSPUSRTMPID=%TPCTBSPUSRTMPID% >> %LOG%
    
    IF %TPCTBSPKEYSID% EQU -1 ( 
    	echo "Error getting TPCTBSPKeys tablespace id" >> %LOG%
    	SET RETCODE=1
    	GOTO EXITWITHSTATUS
    )

    IF %TPCTBSPPMID% EQU -1 ( 
    	echo "Error getting TPCTBSPPM tablespace id" >> %LOG%
    	SET RETCODE=1 
    	GOTO EXITWITHSTATUS
    )

    IF %TPCTBSPTEMPID% EQU -1 ( 
    	echo "Error getting TPCTBSSPTEMP tablespace id" >> %LOG%
    	SET RETCODE=1 
    	GOTO EXITWITHSTATUS
    )

    IF %TPCTBSPDATAID% EQU -1 ( 
    	echo "Error getting TPCTBSPDATA tablespace id" >> %LOG%
    	SET RETCODE=1 
    	GOTO EXITWITHSTATUS
    )

    IF %TPCTBSPUSRTMPID% EQU -1 ( 
    	echo "Error getting TPCTBSPUSRTMP tablespace id" >> %LOG%
    	SET RETCODE=1 
    	GOTO EXITWITHSTATUS
    )


	echo "Restoring database %DB_NAME% ..." >> %LOG%
    if DEFINED TIMESTAMP (
      SET TAKEN_AT=TAKEN AT %TIMESTAMP%
    ) else (
      SET TAKEN_AT=
    )

    echo "%DB2_CMD% RESTORE DATABASE %DB_NAME% USER %DB_USER% USING ******** FROM "%BACKUP_DIR%" %TAKEN_AT% WITH 2 BUFFERS BUFFER 1024 REDIRECT PARALLELISM 1 WITHOUT ROLLING FORWARD WITHOUT PROMPTING" >> %LOG%	
    %DB2_CMD% RESTORE DATABASE %DB_NAME% USER %DB_USER% USING %DB_PASSWORD% FROM \"%BACKUP_DIR%\" %TAKEN_AT% WITH 2 BUFFERS BUFFER 1024 REDIRECT PARALLELISM 1 WITHOUT ROLLING FORWARD WITHOUT PROMPTING >> %LOG%


	REM Set the table spaces for the managed DB2.
	echo "Setting container TPCTBSPKeys..." >> %LOG%
	db2 "set tablespace containers for %TPCTBSPKEYSID% using (path '%DB_USER%\TPCDB\TPC\TPCTBSPKeys')" >> %LOG%
	IF ERRORLEVEL 1 (
		echo "Set table space container for TPCTBSPKeys failed " >> %LOG%
		SET RETCODE=2
		GOTO EXITWITHSTATUS
	) ELSE (
		echo "Set table space container for TPCTBSPKeys succeeded" >> %LOG%
	)


	REM Set tablespace for TPCTBSPPM.
	echo "Setting container TPCTBSPPM..." >> %LOG%
	db2 "set tablespace containers for %TPCTBSPPMID% using (path '%DB_USER%\TPCDB\TPC\TPCTBSPPM')" >> %LOG% 
	IF ERRORLEVEL 1 (
		echo "Set table space container for TPCTBSPPM failed " >> %LOG%
		SET RETCODE=3
		GOTO EXITWITHSTATUS
	) ELSE (
		   echo "Set table space container for TPCTBSPPM succeeded " >> %LOG%
	)

	REM Set tablespace for TPCTBSPTEMP.
	echo "Setting container TPCTBSPTEMP..." >> %LOG%
	db2 "set tablespace containers for %TPCTBSPTEMPID% using (path '%DB_USER%\TPCDB\TPC\TPCTBSPTEMP')" >> %LOG%
	IF ERRORLEVEL 1 (
		echo "Set table space container for TPCTBSPTEMP failed " >> %LOG%
		SET RETCODE=4
		GOTO EXITWITHSTATUS
	) ELSE (
		echo "Set table space container for TPCTBSPTEMP succeeded " >> %LOG%
	)

	REM Set tablespace for TPCTBSPData.
	echo "Setting container TPCTBSPData..." >> %LOG%
	db2 "set tablespace containers for %TPCTBSPDATAID% using (path '%DB_USER%\TPCDB\TPC\TPCTBSPData')" >> %LOG% 
	IF ERRORLEVEL 1 (
		echo "Set table space container for TPCTBSPData failed " >> %LOG%
		SET RETCODE=5
		GOTO EXITWITHSTATUS
	) ELSE (
		echo "Set table space container for TPCTBSPData succeeded " >> %LOG%
	)


	REM Set tablespace for TPCTBSPUSRTMP.
	echo "Settting container TPCTBSPUSRTMP..." >> %LOG%
	db2 "set tablespace containers for %TPCTBSPUSRTMPID% using (path '%DB_USER%\TPCDB\TPC\TPCTBSPUSRTMP')" >> %LOG% 
	IF ERRORLEVEL 1 (
		echo "Set table space container for TPCTBSPUSRTMP failed " >> %LOG%
		SET RETCODE=6
		GOTO EXITWITHSTATUS
	) ELSE (
		echo "Set table space container for TPCTBSPUSRTMP succeeded " >> %LOG%
	)

	REM Run DB2 Restore continue command.
        echo "Continuing restore of database..." >> %LOG% 2>&1
	db2 restore db TPCDB continue >> %LOG%
	IF ERRORLEVEL 1 (
		IF %ERRORLEVEL% EQU 2 (
			echo "Storage Control restore continue succeded (RC=2 but that's ok)" >> %LOG%
			REM Reset the ERRORLEVEL to zero.
			cmd /c "exit /b 0"
		) ELSE (
			echo "Storage Control restore continue failed. ERRORLEVEL=%ERRORLEVEL%" >> %LOG%
			SET RETCODE=7
			GOTO EXITWITHSTATUS
		)
	) ELSE (
		echo "Storage Control restore continue succeeded " >> %LOG%
	)
   
    echo "Restore completed" >> %LOG%



	SET migrationReturnCode=%ERRORLEVEL%
    GOTO %Return%

:CompleteMigration 
    REM connect to db2, unquiesce database
	echo "%DB2_CMD% connect TO %DB_NAME%" >> %LOG%
	%DB2_CMD% CONNECT TO %DB_NAME% USER %DB_USER% USING %DB_PASSWORD% >> %LOG%
	if ERRORLEVEL 1 (
           echo "Error while connecting to the database." >> %LOG% 2>&1
           EXIT /B 1
        )

	echo "Unquiesce database ..." >> %LOG%
	%DB2_CMD% UNQUIESCE DATABASE >> %LOG%
	if ERRORLEVEL 1 (
           echo "Error while unquiesceing the database." >> %LOG% 2>&1
           EXIT /B 1
        )

	%DB2_CMD% CONNECT RESET >> %LOG%
	if ERRORLEVEL 1 (
           echo "Error while resetting the connection to the database." >> %LOG% 2>&1
           EXIT /B 1
        )

    GOTO %Return%


:initializeJava
   REGEDIT /E DIR_REG.REG "HKEY_LOCAL_MACHINE\SOFTWARE\IBM\IBM Director Server\CurrentVersion" 
   
   for /f "usebackq tokens=1,2 delims==" %%a in (`type DIR_REG.REG`) do ( 
     if %%a=="Root" ( 
        SET DIRHOME=%%b
     ) 
   ) 

	GOTO FINALEOF


:Main

  SET Return=CleanupFilesReturn & GOTO CleanupFiles
  :CleanupFilesReturn

  if "%MIGRATION_OPERATION%"=="backup" (
		SET Return=PrepareForMigrationReturn & GOTO PrepareForMigration
  )
  :PrepareForMigrationReturn

  echo "Migration operation is %MIGRATION_OPERATION%"
  if "%MIGRATION_OPERATION%"=="backup" SET Return=BackupDatabaseReturn & GOTO BackupDatabase
  if "%MIGRATION_OPERATION%"=="restore" SET Return=RestoreDatabaseReturn & GOTO RestoreDatabase  
  :BackupDatabaseReturn
  :RestoreDatabaseReturn

  if "%MIGRATION_OPERATION%"=="backup" (
		SET Return=CompleteMigrationReturn & GOTO CompleteMigration
  )
  :CompleteMigrationReturn


  IF %migrationReturnCode% EQU 2 (
     if "%MIGRATION_OPERATION%" EQU "restore" EXIT /B 0  
  )

  IF %migrationReturnCode% NEQ 0 ( 
    EXIT /B %migrationReturnCode%
  )

  EXIT /B 0

:EXITWITHSTATUS
    echo "RETCODE=%RETCODE%" >> %LOG%
	EXIT /B %RETCODE%

:EOF
:FINALEOF
