#!/bin/sh
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2011, 2012 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

# There are 3 scenarios when this script gets called
#
# 1. This script is called by Director framework when customer moves from
#    6.2.1 Director to 6.3. Before running the migration, they run some 
#    pre-config scripts.  During migration from 6.2.1 DB2 to 6.3 embedded DB2
#    we need to run pre-config for DB2. But for windows we haven't seen.
#    For linux we needed to run grant authorities for DB2 user on TPCDB
# 2. When ever Director db2 user password changes, director framework calls
#    with -p option
# 3. When ever Director cfgdbcmd.sh is run.  If the database is being changed
#    Director framework calls with -c <Database Type, DB2, MDB2,SQLServer,
#    Oracle. We return in error if it is not DB2 or MDB2
#
#  This script gets called twice each time.  First with the -c flag to ensure
#  that the database configuration is valid for our advanced manager before
#  any changes are made.  If we return zero to the config check, then database
#  processing continues to make the needed changes.  Once complete, this script
#  is called a second time to notify us that the changes are complete and that
#  we need to update the information for our advanced manager.  In this case,
#  they call us without any parameters, since any variety of options could
#  have changed.

. /opt/ibm/director/StorageControl/bin/initSC.sh

IsConfigCheck="no"
LOG=$DIRHOME/log/SCExt.log

#If we were called for Migration, we need to set up the Admin Authority.
function ChangeMigrationAdmin
{
    DB_USER=$(grep rcs.dbuser $DIRHOME/lwi/conf/overrides/database.properties  | awk '{print $3}')
    DB2HOME=$(cat /etc/passwd | grep $DB_USER | awk -F':' '{print $6}')
    . $DB2HOME/sqllib/db2profile

    PATH=$DB2HOME/sqllib/bin:$PATH
    LIBPATH=$DB2HOME/sqllib/lib64:$LIBPATH
    LD_LIBRARY_PATH=$DB2HOME/sqllib/lib64:$LD_LIBRARY_PATH
    export PATH 
    export LIBPATH 
    export LD_LIBRARY_PATH

    echo "PATH:" $PATH >> $LOG
    echo "DB2HOME:" $DB2_HOME >> $LOG
    echo "LIBPATH:" $LIBPATH >> $LOG
    echo "LD_LIBRARY_PATH:" $LD_LIBRARY_PATH >> $LOG
    echo "DB2INSTANCE:" $DB2INSTANCE >> $LOG
    echo "INSTANCENAME:" $INSTANCENAME >> $LOG

    echo "Granting Authorities" >> $LOG
    su - $DB_USER -c ". /home/${DB_USER}/sqllib/db2profile ; db2set DB2_RESTORE_GRANT_ADMIN_AUTHORITIES=ON" >> $LOG
    if [ $? != 0 ] ; then 
       echo "failed to set DB2_RESTORE_GRANT_ADMIN_AUTHORITIES" >> $LOG
       return 1
    fi
    echo "Granted Authorities" >> $LOG

    su - $DB_USER -c ". /home/${DB_USER}/sqllib/db2profile ; db2stop" >> $LOG 2>&1
    if [ $? != 0 ] ; then 
       echo "failed to stop DB2" >> $LOG
       return 1
    fi
    echo "Stopped DB2 server" >> $LOG
    su - $DB_USER -c ". /home/${DB_USER}/sqllib/db2profile ; db2start" >> $LOG 2>&1
    if [ $? != 0 ] ; then 
       echo "failed to start DB2" >> $LOG
       return 1
    fi
    echo "Started DB2 server" >> $LOG
}


#----------------Start of Mainline Code Execution--------------------

if [ -f $DIRHOME/lwi/conf/overrides/old_database.properties ] ; then

# If there is a old_database.properties that means we are called for migration
   ChangeMigrationAdmin

# In case of error we need to check with Director team, whether we need
# to skip migration

else
#Loop through the command line arguments until we find an empty argument.
#
#The Design here is that when the database properties change, 
#all extension listeners in <DIRHOME>/proddata/dbconfig/ext/ get called.
#1.  For a "config check", they call the extension listeners with the -c flag.
#2.  For a complete config change, they call us with no paramaters.
#3.  When changePassword.sh is run, the extension listener is called with
#the -p flag.
#4.  When Migration is run, the extension listener is called and parameters don't matter.
   until [ -z "$1" ]
   do
      if [ "$1" == "-c" ] ; then
         #The -c flag indicates a configuration check call.  We validate that this
         #database configuration is valid for our advanced manager, but we do not 
         #make changes yet.
         IsConfigCheck="yes"
         break
      fi
      #Shift all the command line args left one.
      shift
   done

   if [ "$IsConfigCheck" == "yes" ] ; then
   echo "Checking TPC DB properties to validate config" >> $LOG
      #Run the CheckDBProperties.java
      "$JAVAPATH/java"  -cp $CLASSPATH CheckDBProperties $*
      if [ $? != 0 ] ; then
         echo "Failed checking properties $*" >> $LOG
         # RC 1 indicates that the requested operation is not supported.
         exit 1
      else
         echo "Successfully checked properties" >> $LOG
      fi
   else
      echo "Changing TPC DB properties" >> $LOG
      #Run the ChangeTPCDB2Properties.java file.
      "$JAVAPATH/java" -cp $CLASSPATH ChangeTPCDB2Properties $* >> $LOG 2>&1
      if [ $? != 0 ] ; then
         echo "Failed changing properties $*" >> $LOG
         # RC 2 indicates that processing failed.
         exit 2
      else
         echo "Successfully changed properties" >> $LOG
      fi
   fi
fi


